@@ -7,16 +7,15 @@ from threading import Thread
 from typing import Union, Callable, Any
 from collections import defaultdict
 
-
 from datetime import datetime
 from dateutil.relativedelta import relativedelta
+from typing import List, Dict
+from colorama import Style, Fore
 import time
 import json
-
-from colorama import Style, Fore
 import logging
-from typing import List, Dict
 
+# import traceback
 
 class TD:
     def __init__(self,
@@ -61,7 +60,8 @@ class TD:
         self.broker_token = broker_token
         self.hist_data = {}
         self.live_data = {}
-        self.min_live_data = {}
+        self.one_min_live_data = {}
+        self.five_min_live_data = {}
         self.symbol_mkt_id_map = defaultdict(set)
         # self.streaming_symbols = {}
         self.touchline_data = {}
@@ -83,6 +83,10 @@ class TD:
 
     def connect_thread(self):
         self.live_websocket.run_forever(ping_interval=10, ping_timeout=5)
+        while not self.live_websocket.disconnect_flag:
+            self.logger.info(f"{Style.BRIGHT}{Fore.RED}Connection dropped due to no network , Attempting reconnect @ {Fore.CYAN}{datetime.now()}{Style.RESET_ALL}")
+            self.live_websocket.reconnect()
+            time.sleep(10)
         self.logger.debug('Goodbye (properly) !!')
 
     def disconnect(self):
@@ -222,6 +226,24 @@ class TD:
         else:
             raise TDLiveDataError("Invalid req_id datatype...")
 
+    
+    def map_subscriptions(self , resolved_contract , req_id):
+        subs = self.live_websocket.subscription_type.split('+')
+        if resolved_contract in self.symbol_mkt_id_map.keys():
+            self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+        if 'tick' in subs :
+            if req_id not in self.live_data.keys() :
+                self.live_data[req_id] = TickLiveData(resolved_contract)
+        if '1min' in subs:
+            if req_id not in self.one_min_live_data.keys() :
+                self.one_min_live_data[req_id] = MinLiveData(resolved_contract)
+        if '5min' in subs:
+            if req_id not in self.five_min_live_data.keys() :
+                self.five_min_live_data[req_id] = MinLiveData(resolved_contract)
+        if 'tick' not in subs:
+            self.live_data[req_id] = MinLiveData(resolved_contract)
+
+
     def start_live_data(self, resolved_contracts, req_id=None):  # TODO: Prevent reuse of req_ids
         symbols_to_call = []
         if req_id is not None:
@@ -235,25 +257,7 @@ class TD:
                 self.touchline_data[req_id] = self.touchline_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
             else:
                 self.touchline_data[req_id] = TouchlineData()
-            if self.live_websocket.subscription_type == 'tick':
-                if resolved_contract in self.symbol_mkt_id_map.keys():
-                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
-                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != TickLiveData:
-                    self.live_data[req_id] = TickLiveData(resolved_contract)
-            elif '+' in self.live_websocket.subscription_type:
-                if resolved_contract in self.symbol_mkt_id_map.keys():
-                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
-                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != TickLiveData:
-                    self.live_data[req_id] = TickLiveData(resolved_contract)
-                if resolved_contract in self.symbol_mkt_id_map.keys():
-                    self.min_live_data[req_id] = self.min_live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
-                elif req_id not in self.min_live_data.keys() or type(self.min_live_data[req_id]) != MinLiveData:
-                    self.min_live_data[req_id] = MinLiveData(resolved_contract)
-            elif 'min' in self.live_websocket.subscription_type:
-                if resolved_contract in self.symbol_mkt_id_map.keys():
-                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
-                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != MinLiveData:
-                    self.live_data[req_id] = MinLiveData(resolved_contract)
+            self.map_subscriptions(resolved_contract , req_id)
             if resolved_contract not in self.symbol_mkt_id_map.keys():
                 symbols_to_call.append(resolved_contract)
             self.symbol_mkt_id_map[resolved_contract].update({req_id})
@@ -287,13 +291,22 @@ class TD:
         self.logger.info(f"Clearing bidask_callback...")
         self.live_websocket.bidask_callback = None
 
-    def bar_callback(self, func: Callable[[TickLiveData], Any]):
-        self.logger.info(f"Defining bar_callback...")
-        self.live_websocket.bar_callback = func
+    def one_min_bar_callback(self, func: Callable):
+        self.logger.info(f"Defining one min bar_callback...")
+        self.live_websocket.one_min_bar_callback = func
+
+    def clear_one_min_bar_callback(self):
+        self.logger.info(f"Clearing one min bar_callback...")
+        self.live_websocket.one_min_bar_callback = None
+
+    def five_min_bar_callback(self, func: Callable) :
+        self.logger.info(f"Defining five min bar_callback...")
+        self.live_websocket.five_min_bar_callback = func
+
+    def clear_five_min_bar_callback(self):
+        self.logger.info(f"Clearing five min bar_callback...")
+        self.live_websocket.five_min_bar_callback = None
 
-    def clear_bar_callback(self):
-        self.logger.info(f"Clearing bar_callback...")
-        self.live_websocket.bar_callback = None
 
     # def get_touchline(self):
     #     self.live_websocket.send(f'{{"method": "touchline"}}')
@@ -304,6 +317,7 @@ class TD:
         try:
             chain = OptionChain(self , symbol, expiry , chain_length , future_price , bid_ask  , market_open_post_hours )
         except Exception as e:
+            # traceback.print_exc()
             print(f'Please check symbol: {symbol} and its expiry: {expiry.date()}')
             exit()
         symb_ids = self.start_live_data(chain.option_symbols)
