@@ -9,6 +9,7 @@ from collections import defaultdict
 
 from colorama import Style, Fore
 
+import traceback
 
 class LiveClient(WebSocketApp):
 
@@ -31,8 +32,10 @@ class LiveClient(WebSocketApp):
         self.disconnect_flag = False
         self.heartbeat_check_thread = Thread(target=self.check_heartbeat, daemon=True)
         self.trade_callback = None
-        self.bar_callback = None
+        self.one_min_bar_callback = None
+        self.five_min_bar_callback = None
         self.bidask_callback = None
+        self.subs = ''
         # if self.parent_app.live_port == 8086 or self.parent_app.live_port == 8084:
         #     # self.heartbeat_check_thread.start()
         #     pass
@@ -89,18 +92,18 @@ class LiveClient(WebSocketApp):
     def on_msg_func(self, *args):
         message = args[-1]
         msg = json.loads(message)
-        if 'message' in msg.keys():
+        msg_keys = msg.keys()
+        # print(msg)
+        if 'message' in msg_keys:
             self.handle_message_data(msg)
-        if 'trade' in msg.keys():
-            trade = msg['trade']
-            self.handle_trade_data(trade)
-        elif 'bidask' in msg.keys():
-            bidask = msg['bidask']
-            self.handle_bid_ask_data(bidask)
-        elif any(['min' in key for key in msg.keys()]):
-            bar_key = next(key for key in msg.keys() if 'min' in key)
-            bar_data = msg[bar_key]
-            self.handle_bar_data(bar_data)
+        if 'trade' in msg_keys:
+            self.handle_trade_data( msg['trade'] )
+        elif 'bidask' in msg_keys:
+            self.handle_bid_ask_data( msg['bidask'] )
+        elif 'bar1min' in msg_keys:
+            self.handle_bar_data( msg['bar1min'] , bar_type= '1min' )
+        elif 'bar5min' in msg_keys:
+            self.handle_bar_data( msg['bar5min'] , bar_type= '5min')
 
     def handle_message_data(self, msg):
         if msg['success']:
@@ -112,10 +115,14 @@ class LiveClient(WebSocketApp):
                 #                  f"as {msg['subscription']}...{Style.RESET_ALL}")
                 self.logger.warning(f"{Style.NORMAL}{Fore.BLUE}Connected successfully to {msg['message']}... {Style.RESET_ALL}")
                 self.subscription_type = msg['subscription']
+                self.subs = self.subscription_type.split('+')
+                # print(self.subscription_type)
             elif msg['message'] in ['symbols added', 'touchline']:
                 self.add_contract_details(msg['symbollist'])
             elif msg['message'] == 'symbols removed':
                 self.remove_symbols(msg['symbollist'])
+            elif msg['message'] == 'marketstatus':
+                self.logger.debug(f"Market status message -> {msg['data']}") 
         else:
             self.logger.error(f"{Style.BRIGHT}{Fore.RED}The request encountered an error - {msg['message']}{Style.RESET_ALL}")
 
@@ -132,7 +139,7 @@ class LiveClient(WebSocketApp):
                 del self.parent_app.live_data[req_id]
                 del self.parent_app.touchline_data[req_id]
                 if '+' in self.subscription_type:
-                    del self.parent_app.min_live_data[req_id]
+                    del self.parent_app.one_min_live_data[req_id]
             del self.parent_app.symbol_mkt_id_map[contract.upper()]
 
     def add_contract_details(self, contracts_list):
@@ -140,9 +147,9 @@ class LiveClient(WebSocketApp):
             if contract_details is not None:
                 self.contract_mapping[int(contract_details[1])] = symbol = contract_details[0]
                 for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
+                    self.parent_app.touchline_data[ticker_id].symbol = symbol
                     self.parent_app.touchline_data[ticker_id].symbol_id = int(contract_details[1])
                     self.parent_app.touchline_data[ticker_id].timestamp = datetime.strptime(contract_details[2], '%Y-%m-%dT%H:%M:%S')
-                    self.parent_app.touchline_data[ticker_id].symbol = symbol
                     self.parent_app.touchline_data[ticker_id].ltp = float(contract_details[3])
                     self.parent_app.touchline_data[ticker_id].ltq = float(contract_details[4])
                     self.parent_app.touchline_data[ticker_id].atp = float(contract_details[5])
@@ -158,9 +165,12 @@ class LiveClient(WebSocketApp):
                     self.parent_app.touchline_data[ticker_id].best_bid_qty = float(contract_details[15])
                     self.parent_app.touchline_data[ticker_id].best_ask_price = float(contract_details[16])
                     self.parent_app.touchline_data[ticker_id].best_ask_qty = float(contract_details[17])
-                    self.parent_app.live_data[ticker_id].populate_using_touchline(self.parent_app.live_data[ticker_id], self.parent_app.touchline_data[ticker_id])
-                    if '+' in self.subscription_type:
-                        self.parent_app.min_live_data[ticker_id].populate_using_touchline(self.parent_app.min_live_data[ticker_id], self.parent_app.touchline_data[ticker_id])
+                    if 'tick' in self.subs:
+                        self.parent_app.live_data[ticker_id].populate_using_touchline(self.parent_app.live_data[ticker_id], self.parent_app.touchline_data[ticker_id])
+                    if '1min' in self.subs:
+                        self.parent_app.one_min_live_data[ticker_id].populate_using_touchline(self.parent_app.one_min_live_data[ticker_id], self.parent_app.touchline_data[ticker_id])
+                    if '5min' in self.subs:
+                        self.parent_app.five_min_live_data[ticker_id].populate_using_touchline(self.parent_app.five_min_live_data[ticker_id], self.parent_app.touchline_data[ticker_id])
                 self.logger.info(f"Updated touchline data for {symbol} and related live data objects...")
             else:
                 self.logger.debug(f'{Style.BRIGHT}{Fore.YELLOW}Probable repeated symbol...{Style.RESET_ALL}')
@@ -237,14 +247,23 @@ class LiveClient(WebSocketApp):
         except Exception as e:
             self.logger.error(f'{Style.BRIGHT}{Fore.RED}Bid-ask feed error encountered ({type(e)})- {e}{Style.RESET_ALL}')
 
-    def handle_bar_data(self, bar_data):
+    def handle_bar_data(self, bar_data , bar_type ):
         try:
             symbol = self.contract_mapping[int(bar_data[0])]
+            ticker_ids =  list(self.parent_app.symbol_mkt_id_map[symbol])
+            if (len(ticker_ids) > 1 ):           #checking multiple id then deepcopying with same touchline data
+                for t_id in ticker_ids[1:]:
+                    if bar_type == '1min':
+                        self.parent_app.one_min_live_data[t_id] = deepcopy(self.parent_app.one_min_live_data[ticker_ids[0]])
+                    else:
+                        self.parent_app.five_min_live_data[t_id] = deepcopy(self.parent_app.five_min_live_data[ticker_ids[0]])
             for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
-                if '+' in self.subscription_type:
-                    obj_to_edit = self.parent_app.min_live_data[ticker_id]
-                else:
-                    obj_to_edit = self.parent_app.live_data[ticker_id]
+                if bar_type == '1min':
+                    obj_to_edit = self.parent_app.one_min_live_data[ticker_id]
+                    obj_to_edit.bar_type = '1min'
+                elif bar_type == '5min':
+                    obj_to_edit = self.parent_app.five_min_live_data[ticker_id]
+                    obj_to_edit.bar_type = '5min' 
                 obj_to_edit.symbol_id = int(bar_data[0])
                 obj_to_edit.timestamp = datetime.strptime(bar_data[1], '%Y-%m-%dT%H:%M:%S')
                 obj_to_edit.symbol = symbol
@@ -258,16 +277,28 @@ class LiveClient(WebSocketApp):
                 obj_to_edit.close = self.parent_app.touchline_data[ticker_id].ltp = float(bar_data[5])
                 obj_to_edit.volume = float(bar_data[6])
                 obj_to_edit.oi = float(bar_data[7])
-                if self.bar_callback:
-                    try:
-                        self.bar_callback(ticker_id, obj_to_edit)
-                    except Exception as e:
-                        self.logger.error(f'{Style.BRIGHT}{Fore.RED}Encountered error with bar_callback - {type(e)} - {e}{Style.RESET_ALL}')
-        except KeyError:
+                # moving data to live_data dict if not subcribed to ticks inorder to work chain
+                if len(self.subs) == 1 :
+                    self.parent_app.live_data[ticker_id] = deepcopy(obj_to_edit)
+                elif len(self.subs) == 2 and 'tick' not in self.subs and bar_type == '1min':
+                    self.parent_app.live_data[ticker_id] = deepcopy(obj_to_edit)
+                
+                if self.one_min_bar_callback and bar_type == '1min' :
+                    self.execute_callbacks( ticker_id , obj_to_edit , '1min')  
+                elif self.five_min_bar_callback and bar_type == '5min' :
+                    self.execute_callbacks( ticker_id , obj_to_edit , '5min' ) 
+        except KeyError as e:
+            # print('error' , e)
             pass
         except Exception as e:
             self.logger.error(f'{Style.BRIGHT}{Fore.RED}Bar feed encountered - {e}{Style.RESET_ALL}')
 
+    def execute_callbacks(self , ticker_id , obj_to_edit , bar_type ):
+        try:
+            self.one_min_bar_callback(ticker_id, obj_to_edit) if bar_type == '1min' else self.five_min_bar_callback(ticker_id, obj_to_edit)
+        except Exception as e:
+            self.logger.error(f'{Style.BRIGHT}{Fore.RED}Encountered error with bar_callback - {type(e)} - {e}{Style.RESET_ALL}')
+
     def ws_error(self, *args):
         error = args[-1]
         if any(isinstance(error, conn_error) for conn_error in [ConnectionResetError, TimeoutError]):
