@@ -1,8 +1,8 @@
-# from datetime import datetime
 from colorama import Style, Fore
 from functools import wraps
 from typing import Union
 from copy import deepcopy
+from datetime import datetime
 
 
 class TouchlineData:
@@ -99,6 +99,10 @@ class TickLiveData:
                 ending_formatter = f"{Style.RESET_ALL}"
             else:
                 starting_formatter = ending_formatter = ""
+        op_dict = deepcopy(self.to_dict())
+        return f"{starting_formatter}{str(op_dict)}{ending_formatter}"
+
+    def to_dict(self):
         op_dict = deepcopy(self.__dict__)
         op_dict['change'] = self.change
         op_dict['change_perc'] = self.change_perc
@@ -109,7 +113,7 @@ class TickLiveData:
         del op_dict['_oi_change']
         del op_dict['_oi_change_perc']
         del op_dict['populate_using_touchline']
-        return f"{starting_formatter}{str(op_dict)}{ending_formatter}"
+        return op_dict
 
     @property
     def change(self):
@@ -176,6 +180,7 @@ class MinLiveData:
         self._change_perc = None
         self._oi_change = None
         self._oi_change_perc = None
+        self.bar_type = None
         self.populate_using_touchline = populate_touchline_data
         # -- Calculated specific
         # --Unused
@@ -193,6 +198,9 @@ class MinLiveData:
         return res
 
     def __str__(self):
+        return str(deepcopy(self.to_dict()))
+
+    def to_dict(self):
         op_dict = deepcopy(self.__dict__)
         op_dict['change'] = self.change
         op_dict['change_perc'] = self.change_perc
@@ -203,7 +211,7 @@ class MinLiveData:
         del op_dict['_oi_change']
         del op_dict['_oi_change_perc']
         del op_dict['populate_using_touchline']
-        return str(op_dict)
+        return op_dict
 
     @property  # TODO: Use decorators for exceptions
     def change(self):
@@ -264,7 +272,6 @@ def populate_touchline_data(data_obj: Union[TickLiveData, MinLiveData], touchlin
         data_obj.close = touchline_obj.ltp
 
 
-
 class TDLiveCalcError(Exception):
     def __str__(self):
         return f"{Style.BRIGHT}{Fore.RED}Something's wrong with the live calculations- {self.args[0]}{Style.RESET_ALL}"
@@ -287,30 +294,35 @@ class TDInvalidRequestError(Exception):
 
 def historical_decorator(func):
     @wraps(func)
-    def dec_helper(obj, contract, end_time, start_time, bar_size, options=None):
-        options = {}
-        if bar_size.lower() == 'tick':
+    def dec_helper(obj, contract, end_time, start_time, bar_size, options=None, bidask=False):
+        if not options:
+            options = {}
+        if bar_size.lower() == 'tick' or bar_size.lower() == 'ticks':
             bar_size = 'tick'
-            options['time_format'] = '%Y-%m-%dT%H:%M:%S'
-            options['processor_to_call'] = obj.hist_tick_data_to_dict_list
+            options['time_format'] = '%Y-%m-%dT%H:%M:%S'  # This is the response format
+            options['processor_to_call'] = {'csv': obj.hist_csv_tick_data_to_dict_list, 'json': obj.hist_json_tick_data_to_dict_list}
         elif bar_size.lower() == 'eod':
-            start_time = start_time.split('T')[0]
-            end_time = end_time.split('T')[0]
-            bar_size = 'EOD'
-            options['time_format'] = '%Y-%m-%d'
-            options['processor_to_call'] = obj.hist_bar_data_to_dict_list
+            # start_time = start_time.split('T')[0]
+            # end_time = end_time.split('T')[0]
+            bar_size = 'eod'
+            options['time_format'] = '%Y-%m-%d'  # This is the response format
+            # options['time_format'] = '%Y-%m-%dT%H:%M:%S'  # This is the response format
+            options['processor_to_call'] = {'csv': obj.hist_csv_bar_data_to_dict_list, 'json': obj.hist_json_bar_data_to_dict_list}
         else:
             bar_size = bar_size.replace(' ', '')
             if bar_size[-1] == 's':
                 bar_size = bar_size[:-1]
-            options['time_format'] = '%Y-%m-%dT%H:%M:%S'
-            options['processor_to_call'] = obj.hist_bar_data_to_dict_list
-        return func(obj, contract, end_time, start_time, bar_size, options)
+            options['time_format'] = '%Y-%m-%dT%H:%M:%S'  # This is the response format
+            options['processor_to_call'] = {'csv': obj.hist_csv_bar_data_to_dict_list, 'json': obj.hist_json_bar_data_to_dict_list}
+        return func(obj, contract, end_time, start_time, bar_size, options, bidask)
+
     return dec_helper
 
-# def historical_reconnect_on_failure(func):
-#     @wraps(func)
-#     def wrapper_func(*args, **kwargs):
-#         try:
-#             return func(*args, **kwargs)
-#         except ConnectionResetError:
+
+def access_token_decorator(func):
+    @wraps(func)
+    def dec_helper(obj, *args, **kwargs):
+        if obj.access_token_expiry_time < datetime.now():
+            obj.hist_login()
+        return func(obj, *args, **kwargs)
+    return dec_helper
\ No newline at end of file
